#!/usr/bin/python
# -*- coding: UTF-8 -*-

############################################################################
##                                                                        ##
## XChat menu                                                             ##
##                                                                        ##
## Copyright (C) 2010  Fabien LOISON (www.flogisoft.com)                  ##
##                                                                        ##
## This program is free software: you can redistribute it and/or modify   ##
## it under the terms of the GNU General Public License as published by   ##
## the Free Software Foundation, either version 3 of the License, or      ##
## (at your option) any later version.                                    ##
##                                                                        ##
## This program is distributed in the hope that it will be useful,        ##
## but WITHOUT ANY WARRANTY; without even the implied warranty of         ##
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          ##
## GNU General Public License for more details.                           ##
##                                                                        ##
## You should have received a copy of the GNU General Public License      ##
## along with this program.  If not, see <http://www.gnu.org/licenses/>.  ##
##                                                                        ##
############################################################################

"""Provides a way to add menu in XChat"""


import xchat


class XChatMenu(object):
    """Provides an XChat menu or submenu"""

    def __init__(self, name, parent=None, sensitive=True, position=-1):
        """The constructor"""
        self._name = name
        self._parent = parent
        if self._parent == None:
            self._path = self._name
        else:
            self._path = "%s/%s" % (self._parent._path, self._name)
            self._parent._children.append(self)
        self._sensitive = sensitive
        self._position = position
        self._children = []
        self._update()

    def set_sensitive(self, sensitive=True):
        """Set if the menu is sensitive or not"""
        self._sensitive = sensitive
        self._update()

    def set_position(self, position):
        """Set the position of the menu"""
        self.remove()
        self._position = position
        self._update()

    def set_name(self, name):
        """Set the name of the menu"""
        self.remove()
        self._name = name
        self._path = name
        self._update()

    def remove(self):
        """Remove the menu"""
        if self._parent != None:
            if self in self._parent._children:
                self._parent._children.remove(self)
        xchat.command("MENU DEL \"%s\"" % self._path)

    def _update(self):
        """Update the menu"""
        cmd = "MENU "
        #Sensitive
        if self._sensitive:
            cmd += "-e1 "
        else:
            cmd += "-e0 "
        #Position
        if self._position >= 0:
            cmd += "-p%i " % self._position
        #
        cmd += "ADD \"%s\"" % self._path
        #Add menu in XChat
        xchat.command(cmd)
        #Update all children
        for child in self._children:
            child._path = "%s/%s" % (self._path, child._name)
            child._update()


class XChatMenuItem(object):
    """Provides an XChat menu item"""

    def __init__(self, name, parent=None, command="", sensitive=True, position=-1):
        """The constructor"""
        self._parent = parent
        self._name = name
        if self._parent == None:
            self._path = self._name
        else:
            self._path = "%s/%s" % (self._parent._path, self._name)
            self._parent._children.append(self)
        self._command = command
        self._sensitive = sensitive
        self._position = position
        self._update()

    def set_sensitive(self, sensitive=True):
        """Set if the item is sensitive or not"""
        self._sensitive = sensitive
        self._update()

    def set_position(self, position):
        """Set the position of the menu"""
        self.remove()
        self._position = position
        self._update()

    def set_name(self, name):
        """Set the name of the item"""
        self.remove()
        self._name = name
        self._path = "%s/%s" % (self._parent._path, self._name)
        self._update()

    def remove(self):
        """Remove the item"""
        if self._parent != None:
            if self in self._parent._children:
                self._parent._children.remove(self)
        xchat.command("MENU DEL \"%s\"" % self._path)

    def _update(self):
        """Update the menu"""
        cmd = "MENU "
        #Sensitive
        if self._sensitive:
            cmd += "-e1 "
        else:
            cmd += "-e0 "
        #Position
        if self._position >= 0:
            cmd += "-p%i " % self._position
        #
        cmd += "ADD \"%s\" \"%s\"" % (self._path, self._command)
        xchat.command(cmd)


class XChatMenuCheckbox(XChatMenuItem):
    """Provides an XChat menu item, with a checkbox"""
    
    def __init__(self, name, parent=None, command_check="", command_uncheck="", checked=False, sensitive=True, position=-1):
        """The constructor"""
        self._parent = parent
        self._name = name
        if self._parent == None:
            self._path = self._name
        else:
            self._path = "%s/%s" % (self._parent._path, self._name)
            self._parent._children.append(self)
        self._command_check = command_check
        self._command_uncheck = command_uncheck
        self._checked = checked
        self._sensitive = sensitive
        self._position = position
        self._update()

    def _update(self):
        """Update the menu"""
        cmd = "MENU "
        #Sensitive
        if self._sensitive:
            cmd += "-e1 "
        else:
            cmd += "-e0 "
        #Position
        if self._position >= 0:
            cmd += "-p%i " % self._position
        #Checked
        if self._checked:
            cmd += "-t1 "
        else:
            cmd += "-t0 "
        #
        cmd += "ADD \"%s\" \"%s\" \"%s\"" % (self._path, self._command_check, self._command_uncheck)
        xchat.command(cmd)


